@@ -6,8 +6,8 @@
 use platform_info::*;
 
 fn main() {
-    let info = PlatformInfo::new().unwrap();
-    // println!("info={:#?}", info);
+    let info = PlatformInfo::new().expect("Unable to determine platform info");
+    // println!("info={:#?}", info); // note: info *may* contain extra platform-specific fields
 
     println!("{}", info.sysname().to_string_lossy());
     println!("{}", info.nodename().to_string_lossy());
