@@ -28,10 +28,9 @@
 // spell-checker:ignore (WinAPI) ctypes CWSTR DWORDLONG dwStrucVersion FARPROC FIXEDFILEINFO HIWORD HMODULE libloaderapi LOWORD LPCSTR LPCVOID LPCWSTR lpdw LPDWORD lplp LPOSVERSIONINFOEXW LPSYSTEM lptstr LPVOID LPWSTR minwindef ntdef ntstatus OSVERSIONINFOEXW processthreadsapi PUINT SMALLBUSINESS SUITENAME sysinfo sysinfoapi sysinfoapi TCHAR TCHARs ULONGLONG VERSIONINFO WCHAR WCHARs winapi winbase winver WSTR wstring
 // spell-checker:ignore (WinOS) ntdll
 
-#![warn(unused_results)]
+#![warn(unused_results)] // enable warnings for unused results
 
 use std::convert::TryFrom;
-use std::error::Error;
 use std::ffi::{OsStr, OsString};
 use std::fmt;
 use std::fmt::{Debug, Formatter};
@@ -42,22 +41,27 @@ use winapi::shared::minwindef::*;
 use winapi::um::sysinfoapi::*;
 use winapi::um::winnt::*;
 
-use crate::PlatformInfoAPI;
+use crate::{PlatformInfoAPI, PlatformInfoError, UNameAPI};
 
 use super::PathStr;
 use super::PathString;
 
+type WinOSError = crate::lib_impl::BoxedThreadSafeStdError;
+
 mod windows_safe;
 use windows_safe::*;
 
 //===
 
 // PlatformInfo
-/// Handles initial retrieval and holds information for the current platform (Windows/WinOS in this case).
+/// Handles initial retrieval and holds cached information for the current platform (Windows/WinOS in this case).
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct PlatformInfo {
+    /// Cached computer name.
     pub computer_name: OsString,
+    /// Wraps a cached [`WinApiSystemInfo`].
     pub system_info: WinApiSystemInfo,
+    /// Wraps a cached [`WinOsVersionInfo`].
     pub version_info: WinOsVersionInfo,
     // * private-use fields
     sysname: OsString,
@@ -68,10 +72,9 @@ pub struct PlatformInfo {
     osname: OsString,
 }
 
-impl PlatformInfo {
-    /// Creates a new instance of `PlatformInfo`.
-    /// Because of the way the information is retrieved, it is possible for this function to fail.
-    pub fn new() -> Result<Self, Box<dyn Error>> {
+impl PlatformInfoAPI for PlatformInfo {
+    // * note: due to the method of information retrieval, this *may* fail
+    fn new() -> Result<Self, PlatformInfoError> {
         let computer_name = WinOsGetComputerName()?;
         let system_info = WinApiSystemInfo(WinAPI_GetNativeSystemInfo());
         let version_info = os_version_info()?;
@@ -98,7 +101,7 @@ impl PlatformInfo {
     }
 }
 
-impl PlatformInfoAPI for PlatformInfo {
+impl UNameAPI for PlatformInfo {
     fn sysname(&self) -> &OsStr {
         &self.sysname
     }
@@ -130,7 +133,7 @@ impl PlatformInfoAPI for PlatformInfo {
 /// Contains information about the current computer system.
 ///
 /// Wraps [SYSTEM_INFO](https://learn.microsoft.com/en-us/windows/win32/api/sysinfoapi/ns-sysinfoapi-system_info).
-#[derive(Clone, Copy /* , Debug, PartialEq, Eq */)]
+#[derive(Clone, Copy /* , Debug, PartialEq, Eq *//* note: implemented elsewhere */)]
 pub struct WinApiSystemInfo(
     // ref: <https://learn.microsoft.com/en-us/windows/win32/api/sysinfoapi/ns-sysinfoapi-system_info> @@ <https://archive.is/cqbrj>
     SYSTEM_INFO,
@@ -276,7 +279,7 @@ impl Eq for WinApiSystemInfo {}
 // WinOSGetComputerName
 /// *Returns* a NetBIOS or DNS name associated with the local computer.
 #[allow(non_snake_case)]
-fn WinOsGetComputerName() -> Result<OsString, Box<dyn Error>> {
+fn WinOsGetComputerName() -> Result<OsString, WinOSError> {
     //## NameType ~ using "ComputerNameDnsHostname" vs "ComputerNamePhysicalDnsHostname"
     // * "ComputerNamePhysicalDnsHostname" *may* have a different (more specific) name when in a DNS cluster
     // * `uname -n` may show the more specific cluster name (see https://clusterlabs.org/pacemaker/doc/deprecated/en-US/Pacemaker/1.1/html/Clusters_from_Scratch/_short_node_names.html)
@@ -300,7 +303,7 @@ fn WinOsGetComputerName() -> Result<OsString, Box<dyn Error>> {
 #[allow(non_snake_case)]
 fn WinOsGetFileVersionInfo<P: AsRef<PathStr>>(
     file_path: P,
-) -> Result<WinApiFileVersionInfo, Box<dyn Error>> {
+) -> Result<WinApiFileVersionInfo, WinOSError> {
     let file_version_size = WinAPI_GetFileVersionInfoSizeW(&file_path);
     if file_version_size == 0 {
         return Err(Box::new(io::Error::last_os_error()));
@@ -316,7 +319,7 @@ fn WinOsGetFileVersionInfo<P: AsRef<PathStr>>(
 // WinOSGetSystemDirectory
 /// *Returns* a resolved path to the Windows System Directory (aka `%SystemRoot%`).
 #[allow(non_snake_case)]
-fn WinOsGetSystemDirectory() -> Result<PathString, Box<dyn Error>> {
+fn WinOsGetSystemDirectory() -> Result<PathString, WinOSError> {
     let required_capacity: UINT = WinAPI_GetSystemDirectoryW(None);
     let mut data = vec![0; usize::try_from(required_capacity)?];
     let result = WinAPI_GetSystemDirectoryW(&mut data);
@@ -337,7 +340,7 @@ fn WinOsGetSystemDirectory() -> Result<PathString, Box<dyn Error>> {
 /// it useless for Windows 8.1 and later windows versions.
 // ref: <https://learn.microsoft.com/en-us/windows/win32/api/sysinfoapi/nf-sysinfoapi-getversionexw> @@ <https://archive.is/bYAwT>
 // ref: <https://learn.microsoft.com/en-us/windows/win32/api/winnt/ns-winnt-osversioninfoexw> @@ <https://archive.is/n4hBb>
-fn os_version_info() -> Result<WinOsVersionInfo, Box<dyn Error>> {
+fn os_version_info() -> Result<WinOsVersionInfo, WinOSError> {
     match os_version_info_from_dll() {
         Ok(os_info) => Ok(os_info),
         Err(_) => {
@@ -352,7 +355,7 @@ fn os_version_info() -> Result<WinOsVersionInfo, Box<dyn Error>> {
 
 // os_version_info_from_dll
 /// *Returns* version info (as [`WinOsVersionInfo`]) obtained via `NTDLL/RtlGetVersion()`.
-fn os_version_info_from_dll() -> Result<WinOsVersionInfo, Box<dyn Error>> {
+fn os_version_info_from_dll() -> Result<WinOsVersionInfo, WinOSError> {
     let os_info = NTDLL_RtlGetVersion()?;
     Ok(WinOsVersionInfo {
         os_name: winos_name(
@@ -372,7 +375,7 @@ fn os_version_info_from_dll() -> Result<WinOsVersionInfo, Box<dyn Error>> {
 /// *Returns* version info (as [`WinOsVersionInfo`]) obtained from `file_path`.
 ///
 /// `file_path` ~ if empty or `None`, default to the full path of "kernel32.dll" (a known, omnipresent, system file)
-fn version_info_from_file<I, P>(file_path: I) -> Result<WinOsVersionInfo, Box<dyn Error>>
+fn version_info_from_file<I, P>(file_path: I) -> Result<WinOsVersionInfo, WinOSError>
 where
     I: Into<Option<P>>,
     P: AsRef<PathStr>,
@@ -390,14 +393,14 @@ where
     info.wProductType = VER_NT_WORKSTATION;
 
     let mask = WinAPI_VerSetConditionMask(0, VER_SUITENAME, VER_EQUAL);
-    let suite_mask = if WinAPI_VerifyVersionInfoW(&info, VER_SUITENAME, mask) != 0 {
+    let suite_mask = if WinAPI_VerifyVersionInfoW(&info, VER_SUITENAME, mask) != FALSE {
         VER_SUITE_WH_SERVER
     } else {
         0
     };
 
     let mask = WinAPI_VerSetConditionMask(0, VER_PRODUCT_TYPE, VER_EQUAL);
-    let product_type = if WinAPI_VerifyVersionInfoW(&info, VER_PRODUCT_TYPE, mask) != 0 {
+    let product_type = if WinAPI_VerifyVersionInfoW(&info, VER_PRODUCT_TYPE, mask) != FALSE {
         VER_NT_WORKSTATION
     } else {
         0
@@ -414,7 +417,7 @@ where
 /// *Returns* version (as an [`MmbrVersion`]) copied from a view (aka slice) into the supplied `file_version_info`.
 fn mmbr_from_file_version(
     file_version_info: WinApiFileVersionInfo,
-) -> Result<MmbrVersion, Box<dyn Error>> {
+) -> Result<MmbrVersion, WinOSError> {
     let info = WinOsFileVersionInfoQuery_root(&file_version_info)?;
     Ok(MmbrVersion {
         major: DWORD::try_from(HIWORD(info.dwProductVersionMS))?,
@@ -496,11 +499,11 @@ fn determine_machine(system_info: &WinApiSystemInfo) -> OsString {
         },
         PROCESSOR_ARCHITECTURE_IA64 => "ia64",
         PROCESSOR_ARCHITECTURE_ARM => "arm", // `arm` may be under-specified compared to GNU implementations
-        PROCESSOR_ARCHITECTURE_ARM64 => "aarch64",
+        PROCESSOR_ARCHITECTURE_ARM64 => "aarch64", // alternatively, `arm64` may be more correct
         PROCESSOR_ARCHITECTURE_MIPS => "mips",
         PROCESSOR_ARCHITECTURE_PPC => "powerpc",
         PROCESSOR_ARCHITECTURE_ALPHA | PROCESSOR_ARCHITECTURE_ALPHA64 => "alpha",
-        PROCESSOR_ARCHITECTURE_SHX => "superh", // "SuperH" processor
+        PROCESSOR_ARCHITECTURE_SHX => "superh", // a "SuperH" processor
         _ => "unknown",
     };
 
@@ -508,7 +511,7 @@ fn determine_machine(system_info: &WinApiSystemInfo) -> OsString {
 }
 
 fn determine_osname(version_info: &WinOsVersionInfo) -> OsString {
-    let mut osname = OsString::from(crate::HOST_OS_NAME);
+    let mut osname = OsString::from(crate::lib_impl::HOST_OS_NAME);
     osname.extend([
         OsString::from(" ("),
         version_info.os_name.clone(),
@@ -583,7 +586,7 @@ fn test_osname() {
     let info = PlatformInfo::new().unwrap();
     let osname = info.osname().to_string_lossy();
     println!("osname=[{}]'{}'", osname.len(), osname);
-    assert!(osname.starts_with(crate::HOST_OS_NAME));
+    assert!(osname.starts_with(crate::lib_impl::HOST_OS_NAME));
 }
 
 #[test]
@@ -724,12 +727,30 @@ fn test_known_winos_names() {
         winos_name(10, 0, 22621, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
         "Windows 11"
     );
+    // test unmatched versions (triggers `_` matches and returns a `default_name`)
+    assert_eq!(
+        winos_name(5, 9, 3790, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows 5.9"
+    );
+    assert_eq!(
+        winos_name(5, 9, 3790, VER_NT_SERVER, VER_SUITE_SMALLBUSINESS),
+        "Windows Server 5.9"
+    );
+    assert_eq!(
+        winos_name(6, 9, 9600, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows 6.9"
+    );
+    assert_eq!(
+        winos_name(6, 9, 9600, VER_NT_SERVER, VER_SUITE_SMALLBUSINESS),
+        "Windows Server 6.9"
+    );
 }
 
 #[test]
 fn structure_clone() {
     let info = PlatformInfo::new().unwrap();
     println!("{:?}", info);
+    #[allow(clippy::redundant_clone)] // ignore `clippy::redundant_clone` warning for direct testing
     let info_copy = info.clone();
     assert_eq!(info_copy, info);
 
@@ -740,6 +761,7 @@ fn structure_clone() {
         release: 4,
     };
     println!("{:?}", mmbr);
+    #[allow(clippy::redundant_clone)] // ignore `clippy::redundant_clone` warning for direct testing
     let mmbr_copy = mmbr.clone();
     assert_eq!(mmbr_copy, mmbr);
 
@@ -747,6 +769,7 @@ fn structure_clone() {
         data: vec![1, 2, 3, 4],
     };
     println!("{:?}", fvi);
+    #[allow(clippy::redundant_clone)] // ignore `clippy::redundant_clone` warning for direct testing
     let fvi_copy = fvi.clone();
     assert_eq!(fvi_copy, fvi);
 }
